<?php
/* vim: set et ts=4 sw=4 sts=4 ai: */
// +----------------------------------------------------------------------+
// | Image post handler                                                   |
// +----------------------------------------------------------------------+
// | @version $Revision: 1.16 $
// +----------------------------------------------------------------------+

//----- Sanity check - this file shouldn't be referenced directly
//		and htaccess should be configured to disallow access to this dir
if(!$page) { exit; }

if($_POST) {
    $errors = array();


    //----- Restricted posting? Allow comments, but any image
    //      upload attempts should be aborted.
    if(FEATURE_RESTRICT_POSTING && $_POST["file"]) {
        $tripcodes = explode(",",FEATURE_RESTRICT_POSTING);
        if(!in_array(tripcode($_POST["tripcode"]),$tripcodes)) {
            $page->fail($ERRORS["restricted"]);
        }
    }

    //----- Check throttling
    if(FEATURE_THROTTLE===true) { 
        $tripcodes = explode(",",FEATURE_THROTTLE_EXCEPTIONS);
        if(!in_array(tripcode($_POST["tripcode"]),$tripcodes)) {
            if ($num=isFlooding()) {
                $page->fail($STRINGS["flood"]);
            }
        }
    }

    //----- Check verification code
    if(FEATURE_VERIFY) {
        $should_be = verification($_SESSION["seed"]);
        $was = $_POST["verify"];
        if(strtolower($should_be) != strtolower($was)) {
            array_push($errors,"Verification mismatch");
        }
    }

    //----- Was parent passed? If so, we're doing a reply
    $parent = is_numeric($_POST["parent"]) ? $_POST["parent"] : false;

    // Name is required
    if($_POST["name"]) {
        $name = $_POST["name"];
        setcookie("name",$_POST["name"],COOKIE_EXPIRES);
    } else {
        array_push($errors,$ERRORS["namerequired"]);
    }

    // Categories - required for thread starters
    $categories = false;
    if(!$parent) {
        if(FEATURE_MULTIPLE_CATEGORY===true && is_array($_POST["categories"]) && count($_POST["categories"])>0) {
            $categories = "/" . implode("/",$_POST["categories"]) . "/";
        } else if($_POST["categories"]) {
            $categories = "/" . $_POST["categories"] . "/";
        }

        if(!$categories) {
            array_push($errors,$ERRORS["categoryrequired"]);
        }
    }

    // Title is not required
    $title = $_POST["title"];

    // Tripcode is not required
    $tripcode = $_POST["tripcode"] ? tripcode($_POST["tripcode"]) : "";
    setcookie("tripcode",$_POST["tripcode"],COOKIE_EXPIRES);

    // Keywords - not required
    $keywords = $_POST["keywords"];

    $image_posted = false;
    // If URL fetching, needs to be valid.
    if ($_POST["url"]) {
        if(!ereg("^http://",$_POST["url"]) && !ereg("^ftp://",$_POST["url"])) {
            array_push($errors,$ERRORS["urlformat"]);
        } else {
            $referrer = ereg_replace("(^.+//[^/]+).+","\\1/",$_POST["url"]);
            $image_posted = true;
        }
    } else if (is_uploaded_file($_FILES['file']['tmp_name'])) {
        $image_posted = true;
        $zip_posted = false;
        // Was it a zip
        $cmd = "file " . $_FILES['file']['tmp_name'];
        exec($cmd,$out);
        if(strstr($out[0],"Zip archive data")) {
            $zip_posted = true;
        } 
        if($zip_posted) {
            $tripcodes = explode(",",FEATURE_ZIP_UPLOADERS);
            if(!in_array(tripcode($_POST["tripcode"]),$tripcodes)) {
                $page->fail($ERRORS["restricted"]);
            }
        }
    }

    // Quit now if errors ...
    if(count($errors)>0) {
        echo "<p>${ERRORS["errorsoccurred"]}</p>\n";
        echo implode("</li>\n<li>",$errors);
        echo "</li></ul>\n";
        $page->fail('');
    }

    $comment = $_POST["comment"];

    // Either an image or a comment is required - empty posts are rejected
    if(!$image_posted && !$comment) {
        $page->fail($ERRORS["emptyreject"]);
    }

    // Get an id for this record
    $date = date("U");
        
    // May alter $parent value if multi-file posting, so store here
    // for use later
    $original_parent = $parent;

    if($image_posted) {

        // If it was a URL fetch, fetch it now
        if($_POST["url"]) {
            $filename = basename($_POST["url"]);
            $filepath = TMP_DIR . "/$filename";

            // Just one file
            $files = array(array($filename,$filepath));

            unset($out);
            // only catch stderr
            $cmd = sprintf(WGET,WGET_USER_AGENT,$referrer,$filepath,$_POST["url"]);
            exec($cmd,$out);
            if(count($out)>0) {
                $page->fail($ERRORS["fetchfail"]);
            }

        // If it was a zip?
        } else if ($zip_posted) {
            // Make a tmp dir
            $tmpdir = uniqid("/tmp/",true);
            if(!@mkdir($tmpdir,0700)) {
                $page->fail($ERRORS["mkdirfail"]);
            }
            unset($out);
            if($zip_posted) {
                $cmd = "unzip -q -d $tmpdir -j " . $_FILES['file']['tmp_name'];
            }
            // Catch stderr
            $cmd .= " 2>&1";
            exec($cmd,$out);
            if(count($out)>0) {
                var_dump($out);
                cleanUpDir($tmpdir);
                $page->fail("Zip processing failed");
            }

            // In PHP5, scandir could be used
            $files = array();
            $d  = opendir($tmpdir);
            while (false !== ($filename = readdir($d))) {
                if($filename != "." && $filename != "..") {
                    array_push($files,array($filename,"$tmpdir/$filename"));
                }
            }
            sort($files);

        // Else... a single image upload
        } else {
            $filename = $_FILES['file']['name'];
            $filepath = $_FILES['file']['tmp_name'];

            // Just one file
            $files = array(array($filename,$filepath));

        }

        include(INCLUDES_DIR . "/process_image.php");

        $i=1;
        foreach($files as $file) {
            $id = getNextId();
            $image = processImage($file[1],$file[0],$id,$parent);

            if($image[0]!==true) {
                if($tmpdir) {
                    cleanUpDir($tmpdir);
                }
                $page->fail($image[1]);
            }

            $image = $image[1];
            if($parent) {
                // No title,keyword,categories
                $result = post($id,$parent,$name,$tripcode,false,$comment,false,false,$image["name"],$image["type"],$image["animated"],$image["size"],"${image["width"]} x ${image["height"]}");
            } else {
                $result = post($id,$parent,$name,$tripcode,$title,$comment,$categories,$keywords,$image["name"],$image["type"],$image["animated"],$image["size"],"${image["width"]} x ${image["height"]}");
            }

            // If multi-file post, and posting to a thread, then set $parent
            // for subsequent entries.
            if($i==1 && $_POST["thread"]) {
                $parent = $id;
            }

            // Unset comment. For single file posts, we already took care oof
            // it. For multi-file posts, it should've only applied to the first.
            $comment = false;

            $i++;
        }

        if($tmpdir) {
            cleanUpDir($tmpdir);
        }

    } else {
        // Just text
        $id = getNextId();
        if($parent) {
            $result = post($id,$parent,$name,$tripcode,false,$comment,false,false);
        } else {
            $result = post($id,false,$name,$tripcode,$title,$comment,$categories,$keywords);
        }
    }

    if(!$result) {
        $page->fail($ERRORS["queryfail"]);
    }

    //----- Bump? If FEATURE_SAGE, then only bump if #comments<FEATURE_SAGE
    if(strtolower($_POST["bump"]) == "on") {
        $bump = true;
        if(is_numeric(FEATURE_SAGE)) {
            $num = getNumComments($original_parent);
            if($num>=FEATURE_SAGE) {
                $bump = false;
            }
        }
        if($bump) {
            bump($original_parent);
        }
    }

    echo $STRINGS["postthanks"];
    echo JS_AUTO_CLOSE;

    $seed = mt_rand();
    $_SESSION["seed"] = $seed;

} else {

    $seed = mt_rand();
    $_SESSION["seed"] = $seed;

    // Find out where we got popped from...
    $referrer = basename($_SERVER["HTTP_REFERER"]);

    // Set up template, and variable filler
    $tmpl = new HTML_Template_IT(TEMPLATE_DIR);
    $tmpl->loadTemplateFile("post.tmpl");
    $vars = array();

    // Is this a thread-starter or a reply?
    $id = $_REQUEST["id"];
    if(is_numeric($id)) {
        
        // REPLY

        $image = getImage($id);
        // Can't comment on comments ;)
        if($image === false || $image["parent"]) {
            $page->fail($ERRORS["queryfail"]);
        }

        $title = $image["title"] ? $image["title"] : $STRINGS["untitled"];
        $title = htmlspecialchars(stripslashes($title));
        $vars["title"] = sprintf($STRINGS["commenttitle"],$title,$image["name"]);
        $vars["id"] = $image["id"];
        if($image["file"]) {
            $vars["thumb_url"] = getImageUrl($id,$image["file"],true);
            $vars["thumb_title"] = $vars["title"];
        }

        $tmpl->touchBlock("bump");

    } else {

        // THREAD-STARTER

        // Categories...
        $categories = getCategories();
        foreach($categories as $row) {
            if ($row["abbr"] == $referrer) {
                $row["checked"] = 'checked="true"';
            }
            $tmpl->setCurrentBlock("category");
            $tmpl->setVariable(array(
                "category_name"     => FEATURE_MULTIPLE_CATEGORY ? "categories[]" : "categories",
                "category_type"     => FEATURE_MULTIPLE_CATEGORY ? "checkbox" : "radio",
                "category_value"    => $row["abbr"],
                "checked"           => $row["checked"],
                "label"             => $row["category"],
            ));
            $tmpl->parseCurrentBlock();
        }
        $tmpl->touchBlock("categories");
        $tmpl->touchBlock("title");
        $tmpl->touchBlock("keywords");
        $tmpl->touchBlock("threaded");

        $vars["title"] = $STRINGS["posttitle"];

    }

    // If FEATURE_RESTRICT_POSTING, then don't show the file upload input
    // on reply forms unless allowed to
    $show_file_upload = false;
    if(!is_numeric($id)) {
        $show_file_upload = true;
    } else {
        if(!FEATURE_RESTRICT_POSTING) {
            $show_file_upload = true;
        } else {
            $tripcodes = explode(",",FEATURE_RESTRICT_POSTING);
            if(in_array(tripcode($_COOKIE["tripcode"]),$tripcodes)) {
                $show_file_upload = true;
            }
        }
    } 

    if($show_file_upload) {
        $tmpl->touchBlock("file_upload");
        $vars["maximum_size"] = MAXIMUM_SIZE;
        $vars["max_bytes"] = round(MAXIMUM_SIZE/1024/1024);

        if(FEATURE_FETCH===true) {
            $tmpl->touchBlock("url_fetch");
        }
    }
        
    // Both thread-starters and replies

    if(FEATURE_VERIFY===true) {
        $vars["seed"] = $seed;
        $tmpl->touchBlock("verify_help");
    }

    $vars["cookie_name"] = $_COOKIE["name"];
    $tmpl->setVariable($vars);
    $tmpl->show();
}
?>
