<?php

/**
 * $file is the tmp file (ie $_FILES['file']['tmp_name'])
 * $name is the name it should have (ie $_FILE['file']['name'])
 * $id is the id the image will have
 * $is_child is boolean whether the file is in a reply or not
 * return array. The return array is one of the following:
 * array(false,$error_message) or
 * array(true,array(
 *	"md5"=>$md5,"width"=>$w,"height"=>$h,"name"=>$newname,"type"=>$type,"size"=>$size))
 * depending on whether the processing succeeeded or not
 */

function processImage($file,$name,$id,$is_child)
{
	global $page;

	$return = array();

	// Check the file type - better be recognized
	$out = "";
	$types = array(
		"JPEG" => "jpg",
		"PNG" => "png",
		"GIF" => "gif");
	exec(IDENTIFY . " \"$file\"",$out);
	$identity = explode(" ",$out[0]);
	$type = $types[$identity[1]];
	if(!$type) {
		$return = array(false,$ERRORS["unknownfile"]);
		return $return;
	}

	// Check size
	$size = filesize($file);
	if($size>MAXIMUM_SIZE) {
		$return = array(false,$ERRORS["toobig"]);
		return $return;
	} 

	// Get md5sum
	if(FEATURE_MD5SUM===true) {
		$md5 = md5_file($file);
	}

	// Was it animated?
	$animated = ereg("\[[0-9]+\]$",$identity[0]) ? 1 : 0;

	// Get dimensions
	$dim = explode("x",$identity[2]);
	$w = $dim[0];
	$h = ereg_replace("\+.+","",$dim[1]);
	
	// Generate a "clean" name
	$newname = ereg_replace("[^-A-z0-9\._]","",stripslashes($name));
	$newname = ereg_replace("\.[^\.]+$","",$newname);
	$newname = sprintf("%06d-%s.%s",$id,$newname,$type);
	
	// Check that target directories exist; else make them
	$imgdir = sprintf("%s/%02d",IMAGE_DIR,floor($id/1000));
	if(!file_exists($imgdir) && !@mkdir($imgdir,0700)) {
		$return = array(false,$ERRORS["mkdirfail"]);
		return $return;
	}
	$thumbdir = sprintf("%s/%02d",THUMB_DIR,floor($id/1000));
	if(!file_exists($thumbdir) && !@mkdir($thumbdir,0700)) {
		$return = array(false,$ERRORS["mkdirfail"]);
		return $return;
	}

	// This is the new file name
	$newfile = "$imgdir/$newname";

	// Move image to image home
	if(is_uploaded_file($file)) {
		if (!@move_uploaded_file($file, $newfile)) {
			$return = array(false,$ERRORS["uploadfail"]);
			return $return;
		}
	} else {
		if(!@rename($file, $newfile)) {
			$return = array(false,$ERRORS["movefail"]);
		return $return;
		}
	} 

	// Make thumbnail. Depending on the aspect ratio, will either hit a target width
	// or a target height, or no scaling if small enough
	$thumbsize = $is_child ? SUBTHUMB_SIZE : THUMB_SIZE;
	if (($w >= $h) && ($w > $thumbsize)) {
		$geometry = "$thumbsize";
	} else if (($h > $w) && ($h > $thumbsize)) {
		$geometry = "x$thumbsize";
	} else {
		// Don't just copy here - animated gifs need to be stripped to 1st frame
		$geometry = "$w";
	}
	$cmd = sprintf(CONVERT,$geometry,$newfile,"$thumbdir/$newname");
	$out = array();
	@exec($cmd,$out);

	// OK, everything worked. Return specs
	$return_data = array(
		"width"=>$w,
		"height"=>$h,
		"name"=>$newname,
		"type"=>$type,
		"animated"=>$animated,
		"size"=>$size);
	if(FEATURE_MD5SUM===true) {
		$return_data["md5"] = $md5;
	}
	return array(true,$return_data);
		
}
?>
